@@ -49,6 +49,7 @@ extern "C" {
 #include "iodevicejpegsourcemanager.h"
 #include "exiv2imageloader.h"
 #include "gwenviewconfig.h"
+#include "imageutils.h"
 
 namespace Gwenview
 {
@@ -113,7 +114,13 @@ struct JpegContent::Private
     // performed a lossy image manipulation, mRawData is cleared and the image
     // pixels are kept in mImage until updateRawDataFromImage() is called.
     QImage mImage;
+
+    // Store the input file, keep it open readOnly. This allows the file to be memory mapped
+    // (i.e. mRawData may point to mFile.map()) rather than completely read on load. Postpone
+    // QFile::readAll() as long as possible (currently in save()).
+    QFile mFile;
     QByteArray mRawData;
+
     QSize mSize;
     QString mComment;
     bool mPendingTransformation;
@@ -205,12 +212,32 @@ JpegContent::~JpegContent()
 
 bool JpegContent::load(const QString& path)
 {
-    QFile file(path);
-    if (!file.open(QIODevice::ReadOnly)) {
+    if (d->mFile.isOpen()) {
+        d->mFile.unmap(reinterpret_cast<unsigned char*>(d->mRawData.data()));
+        d->mFile.close();
+        d->mRawData.clear();
+    }
+
+    d->mFile.setFileName(path);
+    if (!d->mFile.open(QIODevice::ReadOnly)) {
         qCritical() << "Could not open '" << path << "' for reading\n";
         return false;
     }
-    return loadFromData(file.readAll());
+
+    QByteArray rawData;
+    uchar* mappedFile = d->mFile.map(0, d->mFile.size(), QFileDevice::MapPrivateOption);
+    if (mappedFile == nullptr) {
+        // process' mapping limit exceeded, file is sealed or filesystem doesn't support it, etc.
+        qDebug() << "Could not mmap '" << path << "', falling back to QFile::readAll()\n";
+
+        rawData = d->mFile.readAll();
+        // all read in, no need to keep it open
+        d->mFile.close();
+    } else {
+        rawData = QByteArray::fromRawData(reinterpret_cast<char*>(mappedFile), d->mFile.size());
+    }
+
+    return loadFromData(rawData);
 }
 
 bool JpegContent::loadFromData(const QByteArray& data)
@@ -546,6 +573,37 @@ QImage JpegContent::thumbnail() const
         Exiv2::DataBuf thumbnail = d->mExifData.copyThumbnail();
 #endif
         image.loadFromData(thumbnail.pData_, thumbnail.size_);
+
+        Exiv2::ExifData::iterator it = d->mExifData.findKey(Exiv2::ExifKey("Exif.Canon.ThumbnailImageValidArea"));
+        // ensure ThumbnailImageValidArea actually specifies a rectangle, i.e. there must be 4 coordinates
+        if (it != d->mExifData.end() && it->count() == 4) {
+            QRect validArea(QPoint(it->toLong(0), it->toLong(2)), QPoint(it->toLong(1), it->toLong(3)));
+            image = image.copy(validArea);
+        } else {
+            // Unfortunately, Sony does not provide an exif tag that specifies the valid area of the 
+            // embedded thumbnail. Need to derive it from the size of the preview image instead.
+            it = d->mExifData.findKey(Exiv2::ExifKey("Exif.Sony1.PreviewImageSize"));
+            if (it != d->mExifData.end() && it->count() == 2) {
+                const long prevHeight = it->toLong(0);
+                const long prevWidth = it->toLong(1);
+
+                const double scale = prevWidth / image.width();
+
+                // the embedded thumb only needs to be cropped vertically
+                const long validThumbAreaHeight = ceil(prevHeight / scale);
+                const long totalHeightOfBlackArea = image.height() - validThumbAreaHeight;
+                // black bars on top and bottom should be equal in height
+                const long offsetFromTop = totalHeightOfBlackArea / 2;
+
+                const QRect validArea(QPoint(0, offsetFromTop), QSize(image.width(), validThumbAreaHeight));
+                image = image.copy(validArea);
+            }
+        }
+
+        Orientation o = orientation();
+        if (GwenviewConfig::applyExifOrientation() && o != NORMAL && o != NOT_AVAILABLE) {
+            image = image.transformed(ImageUtils::transformMatrix(o));
+        }
     }
     return image;
 }
@@ -575,6 +633,17 @@ void JpegContent::setThumbnail(const QImage& thumbnail)
 
 bool JpegContent::save(const QString& path)
 {
+    // we need to take ownership of the input file's data
+    // if the input file is still open, data is still only mem-mapped
+    if (d->mFile.isOpen()) {
+        // backup the mmap() pointer
+        auto* mappedFile = reinterpret_cast<unsigned char*>(d->mRawData.data());
+        // read the file to memory
+        d->mRawData = d->mFile.readAll();
+        d->mFile.unmap(mappedFile);
+        d->mFile.close();
+    }
+
     QFile file(path);
     if (!file.open(QIODevice::WriteOnly)) {
         d->mErrorString = i18nc("@info", "Could not open file for writing.");
